@@ -5,6 +5,22 @@ from typing import TYPE_CHECKING
 
 from daft.logging import setup_logger
 
+###
+# Set up code coverage for when running code coverage with ray
+###
+if "COV_CORE_SOURCE" in os.environ:
+    try:
+        from pytest_cov.embed import init
+
+        init()
+    except Exception as exc:
+        import sys
+
+        sys.stderr.write(
+            "pytest-cov: Failed to setup subprocess coverage. "
+            "Environ: {!r} "
+            "Exception: {!r}\n".format({k: v for k, v in os.environ.items() if k.startswith("COV_CORE")}, exc)
+        )
 ###
 # Setup logging
 ###
@@ -53,7 +69,10 @@ if not dev_build and not user_opted_out:
 ###
 
 from daft.dataframe import DataFrame
+from daft.datatype import DataType
 from daft.expressions import col, lit
+from daft.series import Series
 from daft.udf import polars_udf, udf
+from daft.viz import register_viz_hook
 
-__all__ = ["DataFrame", "col", "lit", "udf", "polars_udf"]
+__all__ = ["DataFrame", "col", "DataType", "lit", "udf", "polars_udf", "Series", "register_viz_hook"]
