@@ -4,9 +4,6 @@ import warnings
 from dataclasses import dataclass
 from typing import TYPE_CHECKING, Any, Callable, Iterable, TypeVar, Union
 
-import pandas
-import pyarrow as pa
-
 from daft import resource_request
 from daft.api_annotations import DataframePublicAPI
 from daft.context import get_context
@@ -24,6 +21,7 @@ from daft.expressions import Expression, col
 from daft.filesystem import get_filesystem_from_path
 from daft.logical import logical_plan
 from daft.logical.schema import ExpressionList
+from daft.resource_request import ResourceRequest
 from daft.runners.partitioning import (
     PartitionCacheEntry,
     PartitionSet,
@@ -33,11 +31,13 @@ from daft.runners.partitioning import (
     vPartitionSchemaInferenceOptions,
 )
 from daft.runners.pyrunner import LocalPartitionSet
-from daft.types import PythonExpressionType
 from daft.viz import DataFrameDisplay
 
 if TYPE_CHECKING:
     from ray.data.dataset import Dataset as RayDataset
+    import numpy as np
+    import pandas
+    import pyarrow as pa
 
 from daft.logical.field import Field
 from daft.logical.schema import Schema
@@ -53,7 +53,7 @@ def _get_tabular_files_scan(
     """Returns a TabularFilesScan LogicalPlan for a given glob filepath."""
     # Glob the path and return as a DataFrame with a column containing the filepaths
     partition_set_factory = get_context().runner().partition_set_factory()
-    partition_set, filepaths_schema = partition_set_factory.glob_paths(path)
+    partition_set, filepaths_schema = partition_set_factory.glob_paths_details(path)
     cache_entry = get_context().runner().put_partition_set_into_cache(partition_set)
     filepath_plan = logical_plan.InMemoryScan(
         cache_entry=cache_entry,
@@ -84,6 +84,8 @@ def _get_tabular_files_scan(
         source_info=source_info,
         filepaths_child=filepath_plan,
         filepaths_column_name=partition_set_factory.FS_LISTING_PATH_COLUMN_NAME,
+        # Hardcoded for now.
+        num_partitions=len(partition_set),
     )
 
 
@@ -99,6 +101,17 @@ class DataFrame:
         Args:
             plan: LogicalPlan describing the steps required to arrive at this DataFrame
         """
+        if not isinstance(plan, logical_plan.LogicalPlan):
+            if isinstance(plan, dict):
+                raise ValueError(
+                    f"DataFrames should be constructed with a dictionary of columns using `DataFrame.from_pydict`"
+                )
+            if isinstance(plan, list):
+                raise ValueError(
+                    f"DataFrames should be constructed with a list of dictionaries using `DataFrame.from_pylist`"
+                )
+            raise ValueError(f"Expected DataFrame to be constructed with a LogicalPlan, received: {plan}")
+
         self.__plan = plan
         self._result_cache: PartitionCacheEntry | None = None
         self._preview = DataFramePreview(preview_partition=None, dataframe_num_rows=None)
@@ -234,13 +247,17 @@ class DataFrame:
         Returns:
             DataFrame: DataFrame created from list of dictionaries
         """
-        if not data:
-            raise ValueError("Unable to create DataFrame from empty list")
-        return cls.from_pydict(data={header: [row[header] for row in data] for header in data[0]})
+        headers: set[str] = set()
+        for row in data:
+            if not isinstance(row, dict):
+                raise ValueError(f"Expected list of dictionaries of {{column_name: value}}, received: {type(row)}")
+            headers.update(row.keys())
+        headers_ordered = sorted(list(headers))
+        return cls.from_pydict(data={header: [row.get(header, None) for row in data] for header in headers_ordered})
 
     @classmethod
     @DataframePublicAPI
-    def from_pydict(cls, data: dict[str, list | pa.Array]) -> DataFrame:
+    def from_pydict(cls, data: dict[str, list | np.ndarray | pa.Array | pa.ChunkedArray]) -> DataFrame:
         """Creates a DataFrame from a Python dictionary
 
         Example:
@@ -248,41 +265,21 @@ class DataFrame:
 
         Args:
             data: Key -> Sequence[item] of data. Each Key is created as a column, and must have a value that is
-                either a Python list or PyArrow array. Values must be equal in length across all keys.
+                a Python list, Numpy array or PyArrow array. Values must be equal in length across all keys.
 
         Returns:
             DataFrame: DataFrame created from dictionary of columns
         """
+        column_lengths = {key: len(data[key]) for key in data}
+        if len(set(column_lengths.values())) > 1:
+            raise ValueError(
+                f"Expected all columns to be of the same length, but received columns with lengths: {column_lengths}"
+            )
 
-        block_data: dict[str, tuple[ExpressionType, Any]] = {}
-        for header in data:
-            arr = data[header]
-
-            if isinstance(arr, pa.Array) or isinstance(arr, pa.ChunkedArray):
-                expr_type = ExpressionType.from_arrow_type(arr.type)
-                block_data[header] = (expr_type, arr.to_pylist() if ExpressionType.is_py(expr_type) else arr)
-                continue
-
-            try:
-                arrow_type = pa.infer_type(arr)
-            except pa.lib.ArrowInvalid:
-                arrow_type = None
-
-            if arrow_type is None or pa.types.is_nested(arrow_type):
-                found_types = {type(o) for o in data[header]} - {type(None)}
-                block_data[header] = (
-                    (ExpressionType.python_object(), list(arr))
-                    if len(found_types) > 1
-                    else (PythonExpressionType(found_types.pop()), list(arr))
-                )
-                continue
-
-            expr_type = ExpressionType.from_arrow_type(arrow_type)
-            block_data[header] = (expr_type, list(arr) if ExpressionType.is_py(expr_type) else pa.array(arr))
-
-        schema = Schema([Field(header, expr_type) for header, (expr_type, _) in block_data.items()])
+        column_types: dict[str, ExpressionType] = {header: ExpressionType.infer_type(data[header]) for header in data}
+        schema = Schema([Field(header, expr_type) for header, expr_type in column_types.items()])
         data_vpartition = vPartition.from_pydict(
-            data={header: arr for header, (_, arr) in block_data.items()}, schema=schema, partition_id=0
+            data={header: arr for header, arr in data.items()}, schema=schema, partition_id=0
         )
         result_pset = LocalPartitionSet({0: data_vpartition})
 
@@ -681,7 +678,8 @@ class DataFrame:
         """
         assert len(columns) > 0
         projection = logical_plan.Projection(
-            self._plan, self.__column_input_to_expression(columns), custom_resource_request=None
+            self._plan,
+            self.__column_input_to_expression(columns),
         )
         return DataFrame(projection)
 
@@ -724,7 +722,7 @@ class DataFrame:
         """
         names_to_skip = set(names)
         el = ExpressionList([col(e.name) for e in self._plan.schema() if e.name not in names_to_skip])
-        return DataFrame(logical_plan.Projection(self._plan, el, custom_resource_request=None))
+        return DataFrame(logical_plan.Projection(self._plan, el))
 
     @DataframePublicAPI
     def where(self, predicate: Expression) -> DataFrame:
@@ -744,7 +742,7 @@ class DataFrame:
 
     @DataframePublicAPI
     def with_column(
-        self, column_name: str, expr: Expression, resource_request: resource_request.ResourceRequest | None = None
+        self, column_name: str, expr: Expression, resource_request: resource_request.ResourceRequest = ResourceRequest()
     ) -> DataFrame:
         """Adds a column to the current DataFrame with an Expression, equivalent to a ``select``
         with all current columns and the new one
@@ -811,6 +809,19 @@ class DataFrame:
         global_limit = logical_plan.GlobalLimit(local_limit, num=num)
         return DataFrame(global_limit)
 
+    @DataframePublicAPI
+    def count_rows(self) -> int:
+        """Executes the Dataframe to count the number of rows.
+
+        Returns:
+            int: count of the number of rows in this DataFrame.
+        """
+        local_count_op = logical_plan.LocalCount(self._plan)
+        coalease_op = logical_plan.Coalesce(local_count_op, 1)
+        local_sum_op = logical_plan.LocalAggregate(coalease_op, [(Expression._sum(col("count")), "sum")])
+        num_rows = DataFrame(local_sum_op).to_pydict()["count"][0]
+        return num_rows
+
     @DataframePublicAPI
     def repartition(self, num: int, *partition_by: ColumnInputType) -> DataFrame:
         """Repartitions DataFrame to ``num`` partitions
@@ -833,7 +844,6 @@ class DataFrame:
             scheme = logical_plan.PartitionScheme.RANDOM
             exprs: ExpressionList = ExpressionList([])
         else:
-            assert len(partition_by) == 1
             scheme = logical_plan.PartitionScheme.HASH
             exprs = self.__column_input_to_expression(partition_by)
 
@@ -1049,7 +1059,7 @@ class DataFrame:
 
         final_op: logical_plan.LogicalPlan
         if need_final_projection:
-            final_op = logical_plan.Projection(gagg_op, final_schema, custom_resource_request=None)
+            final_op = logical_plan.Projection(gagg_op, final_schema)
         else:
             final_op = gagg_op
 
@@ -1196,6 +1206,25 @@ class DataFrame:
 
         return self
 
+    def __len__(self):
+        """Returns the count of rows when dataframe is materialized.
+        If dataframe is not materialized yet, raises a runtime error.
+
+        Returns:
+            int: count of rows.
+
+        """
+
+        if self._result is not None:
+            return len(self._result)
+
+        message = (
+            "Cannot call len() on an unmaterialized dataframe:"
+            " either materialize your dataframe with df.collect() first before calling len(),"
+            " or use `df.count_rows()` instead which will calculate the total number of rows."
+        )
+        raise RuntimeError(message)
+
     @DataframePublicAPI
     def to_pandas(self) -> pandas.DataFrame:
         """Converts the current DataFrame to a pandas DataFrame.
