@@ -19,7 +19,12 @@ from typing import (
 )
 
 import numpy as np
-import pandas as pd
+
+_PANDAS_AVAILABLE = True
+try:
+    import pandas as pd
+except ImportError:
+    _PANDAS_AVAILABLE = False
 
 _POLARS_AVAILABLE = True
 try:
@@ -123,7 +128,7 @@ class ExpressionExecutor:
                 results = pa.chunked_array([results.to_arrow()])
             elif isinstance(results, np.ndarray):
                 results = pa.chunked_array([pa.array(results)])
-            elif isinstance(results, pd.Series):
+            elif _PANDAS_AVAILABLE and isinstance(results, pd.Series):
                 results = pa.chunked_array([pa.array(results)])
             elif isinstance(results, list):
                 results = pa.chunked_array([pa.array(results)])
@@ -836,11 +841,18 @@ class BaseMethodAccessor:
 
 
 class UrlMethodAccessor(BaseMethodAccessor):
-    def download(self) -> UdfExpression:
-        """Treats each string as a URL, and downloads the bytes contents as a bytes column"""
+    def download(self, max_worker_threads: int = 8) -> UdfExpression:
+        """Treats each string as a URL, and downloads the bytes contents as a bytes column
+
+        Args:
+            max_worker_threads: The maximum number of threads to use for downloading URLs, defaults to 8
+
+        Returns:
+            UdfExpression: a BYTES expression which is the bytes contents of the URL, or None if an error occured during download
+        """
         from daft.udf_library import url_udfs
 
-        return url_udfs.download_udf(self._expr)
+        return url_udfs.download_udf(self._expr, max_worker_threads=max_worker_threads)
 
 
 class StringMethodAccessor(BaseMethodAccessor):
