@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import itertools
 from abc import abstractmethod
-from dataclasses import dataclass
+from dataclasses import asdict, dataclass
 from enum import Enum, IntEnum
 from pprint import pformat
 from typing import Any, Generic, TypeVar
@@ -46,15 +46,19 @@ class LogicalPlan(TreeNode["LogicalPlan"]):
     def schema(self) -> Schema:
         return self._schema
 
-    def resource_request(self) -> ResourceRequest | None:
+    def resource_request(self) -> ResourceRequest:
         """Returns a custom ResourceRequest if one has been attached to this LogicalPlan
 
         Implementations should override this if they allow for customized ResourceRequests.
         """
-        return None
+        return ResourceRequest()
 
     @abstractmethod
-    def required_columns(self) -> set[str]:
+    def required_columns(self) -> list[set[str]]:
+        raise NotImplementedError()
+
+    @abstractmethod
+    def input_mapping(self) -> list[dict[str, str]]:
         raise NotImplementedError()
 
     @abstractmethod
@@ -174,6 +178,12 @@ class LogicalPlan(TreeNode["LogicalPlan"]):
         for k, v in fields_to_print.items():
             if isinstance(v, ExpressionList):
                 v = v.exprs
+            elif isinstance(v, Schema):
+                v = v.to_column_expressions().exprs
+            elif isinstance(v, PartitionSpec):
+                v = asdict(v)
+                if isinstance(v["by"], ExpressionList):
+                    v["by"] = v["by"].exprs
             reduced_types[k] = v
         to_render: list[str] = [f"{self.__class__.__name__}\n"]
         space = "    "
@@ -204,8 +214,11 @@ class TabularFilesScan(UnaryNode):
         columns: list[str] | None = None,
         filepaths_child: LogicalPlan,
         filepaths_column_name: str,
+        num_partitions: int | None = None,
     ) -> None:
-        pspec = PartitionSpec(scheme=PartitionScheme.UNKNOWN, num_partitions=filepaths_child.num_partitions())
+        if num_partitions is None:
+            num_partitions = filepaths_child.num_partitions()
+        pspec = PartitionSpec(scheme=PartitionScheme.UNKNOWN, num_partitions=num_partitions)
         super().__init__(schema, partition_spec=pspec, op_level=OpLevel.PARTITION)
 
         if predicate is not None:
@@ -240,8 +253,11 @@ class TabularFilesScan(UnaryNode):
     def __repr__(self) -> str:
         return self._repr_helper(columns_pruned=len(self._columns) - len(self.schema()), source_info=self._source_info)
 
-    def required_columns(self) -> set[str]:
-        return {self._filepaths_column_name} | self._predicate.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [{self._filepaths_column_name} | self._predicate.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [dict()]
 
     def _local_eq(self, other: Any) -> bool:
         return (
@@ -297,8 +313,11 @@ class InMemoryScan(UnaryNode):
             and self.schema() == other.schema()
         )
 
-    def required_columns(self) -> set[str]:
-        return set()
+    def required_columns(self) -> list[set[str]]:
+        return [set()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [dict()]
 
     def rebuild(self) -> LogicalPlan:
         # if we are rebuilding, this will be cached when this is ran
@@ -345,8 +364,11 @@ class FileWrite(UnaryNode):
     def __repr__(self) -> str:
         return self._repr_helper()
 
-    def required_columns(self) -> set[str]:
-        return self._partition_cols.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._partition_cols.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [dict()]
 
     def _local_eq(self, other: Any) -> bool:
         return (
@@ -391,8 +413,11 @@ class Filter(UnaryNode):
     def __repr__(self) -> str:
         return self._repr_helper(predicate=self._predicate)
 
-    def required_columns(self) -> set[str]:
-        return self._predicate.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._predicate.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [{name: name for name in self.schema().column_names()}]
 
     def _local_eq(self, other: Any) -> bool:
         return isinstance(other, Filter) and self.schema() == other.schema() and self._predicate == other._predicate
@@ -409,7 +434,10 @@ class Projection(UnaryNode):
     """Which columns to keep"""
 
     def __init__(
-        self, input: LogicalPlan, projection: ExpressionList, custom_resource_request: ResourceRequest | None
+        self,
+        input: LogicalPlan,
+        projection: ExpressionList,
+        custom_resource_request: ResourceRequest = ResourceRequest(),
     ) -> None:
         schema = projection.to_schema(input.schema())
         super().__init__(schema, partition_spec=input.partition_spec(), op_level=OpLevel.ROW)
@@ -417,14 +445,17 @@ class Projection(UnaryNode):
         self._projection = projection
         self._custom_resource_request = custom_resource_request
 
-    def resource_request(self) -> ResourceRequest | None:
+    def resource_request(self) -> ResourceRequest:
         return self._custom_resource_request
 
     def __repr__(self) -> str:
         return self._repr_helper(output=self._projection.exprs)
 
-    def required_columns(self) -> set[str]:
-        return self._projection.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._projection.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [self._projection.input_mapping()]
 
     def _local_eq(self, other: Any) -> bool:
         return (
@@ -453,7 +484,7 @@ class Sort(UnaryNode):
         for e in self._sort_by:
             dtype = e.resolve_type(input.schema())
             if dtype in {ExpressionType.null(), ExpressionType.bytes(), ExpressionType.logical()}:
-                raise ExpressionTypeError(f"Cannot sort on expression {e} with type: {e}")
+                raise ExpressionTypeError(f"Cannot sort on expression {e} with type: {dtype}")
 
         if isinstance(descending, bool):
             self._descending = [descending for _ in self._sort_by]
@@ -463,8 +494,11 @@ class Sort(UnaryNode):
     def __repr__(self) -> str:
         return self._repr_helper(sort_by=self._sort_by, desc=self._descending)
 
-    def required_columns(self) -> set[str]:
-        return self._sort_by.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._sort_by.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [{name: name for name in self.schema().column_names()}]
 
     def _local_eq(self, other: Any) -> bool:
         return (
@@ -524,8 +558,12 @@ class Explode(MapPartition[ExplodeOp]):
     def __repr__(self) -> str:
         return self._repr_helper()
 
-    def required_columns(self) -> set[str]:
-        return self._map_partition_op.explode_columns.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._map_partition_op.explode_columns.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        explode_columns = self._map_partition_op.explode_columns.input_mapping().keys()
+        return [{name: name for name in self.schema().column_names() if name not in explode_columns}]
 
     def rebuild(self) -> LogicalPlan:
         return Explode(
@@ -551,8 +589,11 @@ class LocalLimit(UnaryNode):
         assert len(new_children) == 1
         return LocalLimit(new_children[0], self._num)
 
-    def required_columns(self) -> set[str]:
-        return set()
+    def required_columns(self) -> list[set[str]]:
+        return [set()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [{name: name for name in self.schema().column_names()}]
 
     def _local_eq(self, other: Any) -> bool:
         return isinstance(other, LocalLimit) and self.schema() == other.schema() and self._num == self._num
@@ -574,8 +615,11 @@ class GlobalLimit(UnaryNode):
         assert len(new_children) == 1
         return GlobalLimit(new_children[0], self._num)
 
-    def required_columns(self) -> set[str]:
-        return set()
+    def required_columns(self) -> list[set[str]]:
+        return [set()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [{name: name for name in self.schema().column_names()}]
 
     def _local_eq(self, other: Any) -> bool:
         return isinstance(other, GlobalLimit) and self.schema() == other.schema() and self._num == self._num
@@ -584,6 +628,34 @@ class GlobalLimit(UnaryNode):
         return GlobalLimit(input=self._children()[0].rebuild(), num=self._num)
 
 
+class LocalCount(UnaryNode):
+    def __init__(self, input: LogicalPlan) -> None:
+        schema = Schema([Field("count", ExpressionType.integer())])
+        super().__init__(schema, partition_spec=input.partition_spec(), op_level=OpLevel.PARTITION)
+        self._register_child(input)
+
+    def __repr__(self) -> str:
+        return self._repr_helper()
+
+    def copy_with_new_children(self, new_children: list[LogicalPlan]) -> LogicalPlan:
+        assert len(new_children) == 1
+        return LocalCount(new_children[0])
+
+    def required_columns(self) -> list[set[str]]:
+        # HACK: Arbitrarily return the first column in the child to ensure that
+        # at least one column is computed by the optimizer
+        return [{self._children()[0].schema().column_names()[0]}]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return []
+
+    def _local_eq(self, other: Any) -> bool:
+        return isinstance(other, LocalCount) and self.schema() == other.schema()
+
+    def rebuild(self) -> LogicalPlan:
+        return LocalCount(input=self._children()[0].rebuild())
+
+
 class PartitionScheme(Enum):
     UNKNOWN = "UNKNOWN"
     RANGE = "RANGE"
@@ -631,8 +703,11 @@ class Repartition(UnaryNode):
             scheme=self._scheme,
         )
 
-    def required_columns(self) -> set[str]:
-        return self._partition_by.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._partition_by.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [{name: name for name in self.schema().column_names()}]
 
     def _local_eq(self, other: Any) -> bool:
         return (
@@ -674,8 +749,11 @@ class Coalesce(UnaryNode):
             num_partitions=self.num_partitions(),
         )
 
-    def required_columns(self) -> set[str]:
-        return set()
+    def required_columns(self) -> list[set[str]]:
+        return [set()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [{name: name for name in self.schema().column_names()}]
 
     def _local_eq(self, other: Any) -> bool:
         return (
@@ -721,14 +799,20 @@ class LocalAggregate(UnaryNode):
         assert len(new_children) == 1
         return LocalAggregate(new_children[0], agg=self._agg, group_by=self._group_by)
 
-    def required_columns(self) -> set[str]:
-        return self._required_cols
+    def required_columns(self) -> list[set[str]]:
+        return [self._required_cols]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        if self._group_by is not None:
+            return [self._group_by.input_mapping()]
+        else:
+            return []
 
     def _local_eq(self, other: Any) -> bool:
         return (
             isinstance(other, LocalAggregate)
             and self.schema() == other.schema()
-            and self._agg == other._agg
+            and all(l[0].is_eq(r[0]) and l[1] == r[1] for l, r in zip(self._agg, other._agg))
             and self._group_by == other._group_by
         )
 
@@ -759,8 +843,11 @@ class LocalDistinct(UnaryNode):
         assert len(new_children) == 1
         return LocalDistinct(new_children[0], group_by=self._group_by)
 
-    def required_columns(self) -> set[str]:
-        return self._group_by.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._group_by.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [self._group_by.input_mapping()]
 
     def _local_eq(self, other: Any) -> bool:
         return (
@@ -786,7 +873,10 @@ class HTTPRequest(LogicalPlan):
     def __repr__(self) -> str:
         return self._repr_helper()
 
-    def required_columns(self) -> set[str]:
+    def required_columns(self) -> list[set[str]]:
+        raise NotImplementedError()
+
+    def input_mapping(self) -> list[dict[str, str]]:
         raise NotImplementedError()
 
     def _local_eq(self, other: Any) -> bool:
@@ -814,7 +904,10 @@ class HTTPResponse(UnaryNode):
     def __repr__(self) -> str:
         return self._repr_helper()
 
-    def required_columns(self) -> set[str]:
+    def required_columns(self) -> list[set[str]]:
+        raise NotImplementedError()
+
+    def input_mapping(self) -> list[dict[str, str]]:
         raise NotImplementedError()
 
     def _local_eq(self, other: Any) -> bool:
@@ -872,9 +965,9 @@ class Join(BinaryNode):
             raise NotImplementedError()
         elif how == JoinType.INNER:
             num_partitions = max(left.num_partitions(), right.num_partitions())
-            right_id_set = self._right_on.to_name_set()
+            right_drop_set = {r.name() for l, r in zip(left_on, right_on) if l.name() == r.name()}
             left_columns = left.schema().to_column_expressions()
-            right_columns = ExpressionList([col(f.name) for f in right.schema() if f.name not in right_id_set])
+            right_columns = ExpressionList([col(f.name) for f in right.schema() if f.name not in right_drop_set])
             unioned_expressions = left_columns.union(right_columns, rename_dup="right.")
             self._left_columns = left_columns
             self._right_columns = ExpressionList(unioned_expressions.exprs[len(self._left_columns.exprs) :])
@@ -914,8 +1007,11 @@ class Join(BinaryNode):
         assert len(new_children) == 2
         return Join(new_children[0], new_children[1], left_on=self._left_on, right_on=self._right_on, how=self._how)
 
-    def required_columns(self) -> set[str]:
-        return self._left_on.required_columns() | self._right_on.required_columns()
+    def required_columns(self) -> list[set[str]]:
+        return [self._left_on.required_columns(), self._right_on.required_columns()]
+
+    def input_mapping(self) -> list[dict[str, str]]:
+        return [self._left_columns.input_mapping(), self._right_columns.input_mapping()]
 
     def _local_eq(self, other: Any) -> bool:
         return (
