@@ -221,20 +221,22 @@ class vPartition:
         return vPartition(columns=tiles, partition_id=partition_id)
 
     @classmethod
-    def from_pydict(cls, data: dict[str, list[Any]], schema: Schema, partition_id: PartID) -> vPartition:
+    def from_pydict(
+        cls, data: dict[str, list[Any] | np.ndarray | pa.Array | pa.ChunkedArray], schema: Schema, partition_id: PartID
+    ) -> vPartition:
         fields = schema.fields
         tiles = {}
         for f in fields.values():
-
+            # Coerce the column data into a list or PyArrow array depending on the provided schema
             col_name = f.name
             col_type = f.dtype
-            col_data = data[col_name]
-
+            col_data: list | pa.Array
             if ExpressionType.is_py(col_type):
-                col_data = list(col_data)
+                col_data = list(data[col_name])
+            elif isinstance(data[col_name], pa.Array) or isinstance(data[col_name], pa.ChunkedArray):
+                col_data = data[col_name]
             else:
-                if not isinstance(col_data, pa.Array) and not isinstance(col_data, pa.ChunkedArray):
-                    col_data = pa.array(col_data, type=col_type.to_arrow_type())
+                col_data = pa.array(data[col_name], type=col_type.to_arrow_type())
 
             block = DataBlock.make_block(col_data)
             tiles[col_name] = PyListTile(column_name=col_name, partition_id=partition_id, block=block)
@@ -566,6 +568,9 @@ class vPartition:
         left_key_ids = list(left_key_part.columns.keys())
 
         right_key_part = right.eval_expression_list(right_on)
+
+        right_key_ids = list(right_key_part.columns.keys())
+
         left_key_list = [left_key_part.columns[le.name()].block for le in left_on]
         right_key_list = [right_key_part.columns[re.name()].block for re in right_on]
 
@@ -583,21 +588,39 @@ class vPartition:
         joined_block_idx = 0
         result_columns = {}
         for k in left_key_ids:
+            assert k not in result_columns
             result_columns[k] = left_key_part.columns[k].replace_block(block=joined_blocks[joined_block_idx])
+
             joined_block_idx += 1
 
         for k in left_nonjoin_ids:
+            assert k not in result_columns
             result_columns[k] = self.columns[k].replace_block(block=joined_blocks[joined_block_idx])
             joined_block_idx += 1
 
+        for lk, rk in zip(left_key_ids, right_key_ids):
+            if lk != rk:
+                while rk in result_columns:
+                    rk = f"right.{rk}"
+
+                assert rk not in result_columns
+                result_columns[rk] = PyListTile(
+                    column_name=rk, partition_id=self.partition_id, block=result_columns[lk].block
+                )
+
         for k in right_nonjoin_ids:
-            result_columns[k] = right.columns[k].replace_block(block=joined_blocks[joined_block_idx])
+
+            while k in result_columns:
+                k = f"right.{k}"
+            assert k not in result_columns
+            result_columns[k] = PyListTile(
+                column_name=k, partition_id=self.partition_id, block=joined_blocks[joined_block_idx]
+            )
             joined_block_idx += 1
 
         assert joined_block_idx == len(result_keys)
-
-        output = vPartition(columns=result_columns, partition_id=self.partition_id)
-        return output.eval_expression_list(output_projection)
+        output_ordering = output_projection.to_column_expressions()
+        return vPartition(columns=result_columns, partition_id=self.partition_id).eval_expression_list(output_ordering)
 
     def _to_file(
         self,
@@ -807,22 +830,6 @@ class PartitionSetFactory(Generic[PartitionT]):
             ]
         )
 
-    @abstractmethod
-    def glob_paths(
-        self,
-        source_path: str,
-    ) -> tuple[PartitionSet[PartitionT], Schema]:
-        """Globs the specified filepath to construct a PartitionSet of file or dir paths
-
-        Args:
-            source_path (str): path to glob
-
-        Returns:
-            PartitionSet[PartitionT]: PartitionSet containing the listings' paths
-            ExpressionList: Schema of the PartitionSet that was constructed
-        """
-        raise NotImplementedError()
-
     @abstractmethod
     def glob_paths_details(
         self,
