@@ -1,7 +1,7 @@
 use std::sync::Arc;
 
 use crate::datatypes::{
-    BooleanArray, DaftDataType, DaftNumericType, DataType, Field, Utf8Array, Utf8Type,
+    BinaryArray, BooleanArray, DaftDataType, DaftNumericType, DataType, Field, Utf8Array, Utf8Type,
 };
 
 use crate::array::DataArray;
@@ -46,6 +46,17 @@ impl From<(&str, &[bool])> for BooleanArray {
     }
 }
 
+impl From<(&str, arrow2::array::BooleanArray)> for BooleanArray {
+    fn from(item: (&str, arrow2::array::BooleanArray)) -> Self {
+        let (name, arrow_array) = item;
+        DataArray::new(
+            Field::new(name, DataType::Boolean).into(),
+            arrow_array.arced(),
+        )
+        .unwrap()
+    }
+}
+
 impl<T: AsRef<str>> From<(&str, &[T])> for DataArray<Utf8Type> {
     fn from(item: (&str, &[T])) -> Self {
         let (name, slice) = item;
@@ -54,6 +65,18 @@ impl<T: AsRef<str>> From<(&str, &[T])> for DataArray<Utf8Type> {
     }
 }
 
+impl From<(&str, &[u8])> for BinaryArray {
+    fn from(item: (&str, &[u8])) -> Self {
+        let (name, slice) = item;
+        let arrow_array = arrow2::array::BinaryArray::<i64>::from_slice([slice]);
+        DataArray::new(
+            Field::new(name, DataType::Binary).into(),
+            arrow_array.arced(),
+        )
+        .unwrap()
+    }
+}
+
 impl<T: DaftDataType> TryFrom<(&str, Box<dyn arrow2::array::Array>)> for DataArray<T> {
     type Error = DaftError;
 
