@@ -2,7 +2,7 @@ use arrow2::array;
 
 use crate::{
     array::{BaseArray, DataArray},
-    datatypes::{DaftNumericType, Utf8Array},
+    datatypes::{BooleanArray, DaftNumericType, Utf8Array},
 };
 
 impl<T> DataArray<T>
@@ -21,3 +21,10 @@ impl Utf8Array {
         self.data().as_any().downcast_ref().unwrap()
     }
 }
+
+impl BooleanArray {
+    // downcasts a DataArray<T> to an Arrow BooleanArray.
+    pub fn downcast(&self) -> &array::BooleanArray {
+        self.data().as_any().downcast_ref().unwrap()
+    }
+}
