@@ -18,7 +18,7 @@ pub enum Expr {
         left: ExprRef,
         right: ExprRef,
     },
-
+    Cast(ExprRef, DataType),
     Column(Arc<str>),
     Literal(lit::LiteralValue),
 }
@@ -40,11 +40,16 @@ impl Expr {
         Expr::Alias(self.clone().into(), name.into())
     }
 
+    pub fn cast(&self, dtype: &DataType) -> Self {
+        Expr::Cast(self.clone().into(), dtype.clone())
+    }
+
     pub fn to_field(&self, schema: &Schema) -> DaftResult<Field> {
         use Expr::*;
 
         match self {
             Alias(expr, name) => Ok(Field::new(name.as_ref(), expr.get_type(schema)?)),
+            Cast(expr, dtype) => Ok(Field::new(expr.name()?, dtype.clone())),
             Column(name) => Ok(schema.get_field(name).cloned()?),
             Literal(value) => Ok(Field::new("literal", value.get_type())),
 
@@ -60,16 +65,38 @@ impl Expr {
                     | Operator::Or => {
                         Field::new(left.to_field(schema)?.name.as_str(), DataType::Boolean)
                     }
-                    _ => Field::new(
-                        left.to_field(schema)?.name.as_str(),
-                        try_get_supertype(&left.get_type(schema)?, &right.get_type(schema)?)?,
-                    ),
+                    Operator::TrueDivide => {
+                        Field::new(left.to_field(schema)?.name.as_str(), DataType::Float64)
+                    }
+                    _ => {
+                        let left_field = left.to_field(schema)?;
+                        let right_field = right.to_field(schema)?;
+                        Field::new(
+                            left_field.name.as_str(),
+                            try_get_supertype(&left_field.dtype, &right_field.dtype)?,
+                        )
+                    }
                 };
                 Ok(result)
             }
         }
     }
 
+    pub fn name(&self) -> DaftResult<&str> {
+        use Expr::*;
+        match self {
+            Alias(.., name) => Ok(name.as_ref()),
+            Cast(expr, ..) => expr.name(),
+            Column(name) => Ok(name.as_ref()),
+            Literal(..) => Ok("literal"),
+            BinaryOp {
+                op: _,
+                left,
+                right: _,
+            } => left.name(),
+        }
+    }
+
     pub fn get_type(&self, schema: &Schema) -> DaftResult<DataType> {
         Ok(self.to_field(schema)?.dtype)
     }
@@ -93,6 +120,7 @@ impl Display for Expr {
                 write_out_expr(f, right)?;
                 Ok(())
             }
+            Cast(expr, dtype) => write!(f, "cast({expr} AS {dtype})"),
             Column(name) => write!(f, "col({name})"),
             Literal(val) => write!(f, "lit({val})"),
         }
@@ -111,7 +139,6 @@ pub enum Operator {
     Plus,
     Minus,
     Multiply,
-    Divide,
     TrueDivide,
     FloorDivide,
     Modulus,
@@ -133,9 +160,8 @@ impl Display for Operator {
             Plus => "+",
             Minus => "-",
             Multiply => "*",
-            Divide => "//",
             TrueDivide => "/",
-            FloorDivide => "floor_div",
+            FloorDivide => "//",
             Modulus => "%",
             And => "&",
             Or => "|",
