@@ -1,4 +1,5 @@
 use pyo3::prelude::*;
+mod datatype;
 mod error;
 mod expr;
 mod series;
@@ -8,6 +9,7 @@ pub fn register_modules(_py: Python, parent: &PyModule) -> PyResult<()> {
     parent.add_class::<expr::PyExpr>()?;
     parent.add_class::<table::PyTable>()?;
     parent.add_class::<series::PySeries>()?;
+    parent.add_class::<datatype::PyDataType>()?;
 
     parent.add_wrapped(wrap_pyfunction!(expr::col))?;
     parent.add_wrapped(wrap_pyfunction!(expr::lit))?;
