@@ -4,30 +4,21 @@ import numpy as np
 import pyarrow as pa
 import pytest
 
-from daft.expressions import ColID, Expression, col
+from daft.expressions import Expression, col
 from daft.logical.schema import ExpressionList
 from daft.runners.blocks import ArrowDataBlock, DataBlock
 from daft.runners.partitioning import PyListTile, vPartition
 
 
-def resolve_expr(expr: Expression) -> Expression:
-    for c in expr.required_columns():
-        c._assign_id(strict=False)
-    expr._assign_id(strict=False)
-    return expr
-
-
 def test_vpartition_eval_expression() -> None:
     expr = (col("a") + col("b")).alias("c")
-    expr = resolve_expr(expr)
     tiles = {}
     for c in expr.required_columns():
         block = DataBlock.make_block(np.ones(10))
-        tiles[c.get_id()] = PyListTile(column_id=c.get_id(), column_name=c.name(), partition_id=0, block=block)
+        tiles[c] = PyListTile(column_name=c, partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     result_tile = part.eval_expression(expr=expr)
     assert result_tile.partition_id == 0
-    assert result_tile.column_id == expr.get_id()
     assert result_tile.column_name == expr.name()
     assert result_tile.block == DataBlock.make_block(np.ones(10) * 2)
 
@@ -36,16 +27,14 @@ def test_vpartition_eval_expression_list() -> None:
     list_of_expr: list[Expression] = []
     # we do this to have the same col ids for column access
     base_expr = col("a") + col("b")
-    base_expr = resolve_expr(base_expr)
     for i in range(4):
         expr = (base_expr + i).alias(f"c_{i}")
-        expr = resolve_expr(expr)
         list_of_expr.append(expr)
     expr_list = ExpressionList(list_of_expr)
     tiles = {}
     for c in expr.required_columns():
         block = DataBlock.make_block(np.ones(10))
-        tiles[c.get_id()] = PyListTile(column_id=c.get_id(), column_name=c.name(), partition_id=0, block=block)
+        tiles[c] = PyListTile(column_name=c, partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     assert len(part) == 10
 
@@ -55,10 +44,9 @@ def test_vpartition_eval_expression_list() -> None:
     assert len(result_vpart.columns) == 4
     for i in range(4):
         expr = list_of_expr[i]
-        col_id = expr.get_id()
-        result_tile = result_vpart.columns[col_id]
+        col_name = expr.name()
+        result_tile = result_vpart.columns[col_name]
         assert result_vpart.partition_id == 0
-        assert result_tile.column_id == expr.get_id()
         assert result_tile.column_name == expr.name()
         assert result_tile.block == DataBlock.make_block((np.ones(10) * 2) + i)
 
@@ -67,7 +55,7 @@ def test_vpartition_to_arrow_table() -> None:
     tiles = {}
     for i in range(4):
         block = DataBlock.make_block(np.ones(10) * i)
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     arrow_table = pa.Table.from_pandas(part.to_pandas())
     assert arrow_table.column_names == [f"col_{i}" for i in range(4)]
@@ -78,12 +66,11 @@ def test_vpartition_to_arrow_table() -> None:
 
 def test_vpartition_from_arrow_table() -> None:
     arrow_table = pa.table([np.ones(10) * i for i in range(4)], names=[f"col_{i}" for i in range(4)])
-    vpart = vPartition.from_arrow_table(arrow_table, column_ids=[ColID(i) for i in range(4)], partition_id=0)
+    vpart = vPartition.from_arrow_table(arrow_table, partition_id=0)
     assert len(vpart) == 10
-    for i, (col_id, tile) in enumerate(vpart.columns.items()):
+    for i, (col_name, tile) in enumerate(vpart.columns.items()):
         assert tile.block == DataBlock.make_block(data=np.ones(10) * i)
-        assert tile.column_id == col_id
-        assert tile.column_name == f"col_{i}"
+        assert tile.column_name == col_name
         assert tile.partition_id == 0
 
 
@@ -91,7 +78,7 @@ def test_vpartition_uneven_tiles() -> None:
     tiles = {}
     for i in range(4):
         block = DataBlock.make_block(np.ones(10 + i) * i)
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
 
     with pytest.raises(ValueError):
         part = vPartition(columns=tiles, partition_id=0)
@@ -101,7 +88,7 @@ def test_vpartition_not_same_partition() -> None:
     tiles = {}
     for i in range(4):
         block = DataBlock.make_block(np.ones(10) * i)
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=i, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=i, block=block)
 
     with pytest.raises(ValueError):
         part = vPartition(columns=tiles, partition_id=0)
@@ -111,7 +98,7 @@ def test_vpartition_wrong_col_id() -> None:
     tiles = {}
     for i in range(4):
         block = DataBlock.make_block(np.ones(10) * i)
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=i + i, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=i + i, block=block)
 
     with pytest.raises(ValueError):
         part = vPartition(columns=tiles, partition_id=0)
@@ -121,7 +108,7 @@ def test_vpartition_head() -> None:
     tiles = {}
     for i in range(4):
         block = DataBlock.make_block(np.ones(10) * i)
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     part = part.head(3)
     arrow_table = pa.Table.from_pandas(part.to_pandas())
@@ -135,7 +122,7 @@ def test_vpartition_sample() -> None:
     tiles = {}
     for i in range(4):
         block = DataBlock.make_block(np.ones(10) * i)
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     part = part.sample(3)
     arrow_table = pa.Table.from_pandas(part.to_pandas())
@@ -146,44 +133,40 @@ def test_vpartition_sample() -> None:
 
 
 def test_vpartition_filter() -> None:
-    expr = col("x") < 4
-    expr = resolve_expr(expr)
+    expr = col("col_0") < 4
 
     # Need to make sure there are no conflicts with column ID of `expr`
-    col_ids = [expr.required_columns()[0].get_id()] + list(range(expr.get_id() + 1, expr.get_id() + 4))
 
     tiles = {}
-    for i in col_ids:
+    for i in range(4):
         block = DataBlock.make_block(np.arange(0, 10, 1))
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
 
     part = vPartition(columns=tiles, partition_id=0)
     part = part.filter(ExpressionList([expr]))
     arrow_table = pa.Table.from_pandas(part.to_pandas())
-    assert arrow_table.column_names == [f"col_{i}" for i in col_ids]
+    assert arrow_table.column_names == [f"col_{i}" for i in range(4)]
 
     for i in range(4):
         assert np.all(arrow_table[i].to_numpy() < 4)
 
 
 def test_vpartition_sort() -> None:
-    expr = col("x")
-    expr = resolve_expr(expr)
+    expr = col("col_0")
 
     tiles = {}
-    col_id = expr.required_columns()[0].get_id()
 
-    for i in range(col_id, col_id + 4):
-        if i == col_id:
+    for i in range(4):
+        if i == 0:
             block = DataBlock.make_block(-np.arange(0, 10, 1))
         else:
             block = DataBlock.make_block(np.arange(0, 10, 1))
 
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     part = part.sort(ExpressionList([expr]))
     arrow_table = pa.Table.from_pandas(part.to_pandas())
-    assert arrow_table.column_names == [f"col_{i}" for i in range(col_id, col_id + 4)]
+    assert arrow_table.column_names == [f"col_{i}" for i in range(4)]
 
     is_sorted = lambda a: np.all(a[:-1] <= a[1:])
     assert is_sorted(arrow_table[0].to_numpy())
@@ -192,23 +175,21 @@ def test_vpartition_sort() -> None:
 
 
 def test_vpartition_sort_desc() -> None:
-    expr = col("x")
-    expr = resolve_expr(expr)
+    expr = col("col_0")
 
     tiles = {}
-    col_id = expr.required_columns()[0].get_id()
 
-    for i in range(col_id, col_id + 4):
-        if i == col_id:
+    for i in range(4):
+        if i == 0:
             block = DataBlock.make_block(-np.arange(0, 10, 1))
         else:
             block = DataBlock.make_block(np.arange(0, 10, 1))
 
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     part = part.sort(ExpressionList([expr]), descending=[True])
     arrow_table = pa.Table.from_pandas(part.to_pandas())
-    assert arrow_table.column_names == [f"col_{i}" for i in range(col_id, col_id + 4)]
+    assert arrow_table.column_names == [f"col_{i}" for i in range(4)]
 
     is_sorted = lambda a: np.all(a[:-1] <= a[1:])
     assert is_sorted(arrow_table[0].to_numpy()[::-1])
@@ -221,7 +202,7 @@ def test_split_by_index_even(n) -> None:
     tiles = {}
     for i in range(0, 4):
         block = DataBlock.make_block(np.arange(0, 100, 1))
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
     part = vPartition(columns=tiles, partition_id=0)
     new_parts = part.split_by_index(n, DataBlock.make_block(data=np.arange(0, 100, 1) % n))
     assert len(new_parts) == n
@@ -240,19 +221,16 @@ def test_split_by_index_even(n) -> None:
 
 @pytest.mark.parametrize("n", [1, 2, 3, 4, 5, 10])
 def test_hash_partition(n) -> None:
-    expr = col("x")
-    expr = resolve_expr(expr)
 
     tiles = {}
-    col_id = expr.required_columns()[0].get_id()
 
-    for i in range(col_id, col_id + 4):
+    for i in range(4):
         block = DataBlock.make_block(np.arange(0, 2 * n, 1) % n)
 
-        tiles[i] = PyListTile(column_id=i, column_name=f"col_{i}", partition_id=0, block=block)
+        tiles[f"col_{i}"] = PyListTile(column_name=f"col_{i}", partition_id=0, block=block)
 
     part = vPartition(columns=tiles, partition_id=0)
-    new_parts = part.split_by_hash(ExpressionList([expr]), n)
+    new_parts = part.split_by_hash(ExpressionList([col("col_0")]), n)
     values_seen = set()
     for i, new_part in enumerate(new_parts):
         assert new_part.partition_id == i
@@ -269,7 +247,7 @@ def test_hash_partition(n) -> None:
 
 def test_partition_quantiles_small_sample_large_partitions():
     data = [1, 2, 3]
-    partition = vPartition({1: PyListTile(1, "foo", 1, ArrowDataBlock(pa.chunked_array([data])))}, 1)
+    partition = vPartition({"foo": PyListTile("foo", 1, ArrowDataBlock(pa.chunked_array([data])))}, 1)
     quantiled_partition = partition.quantiles(10)
-    quantile_boundaries = quantiled_partition.columns[1].block.data.to_pylist()
+    quantile_boundaries = quantiled_partition.columns["foo"].block.data.to_pylist()
     assert sorted(quantile_boundaries) == quantile_boundaries, "quantile boundaries should be in sorted order"
